/* GDK - The GIMP Drawing Kit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 * Copyright (C) 1999 Tor Lillqvist
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#include "config.h"

#include "gdkinput.h"
#include "gdkprivate-directfb.h"
#include "gdkinput-directfb.h"
#include "gdkkeysyms.h"

GdkModifierType _gdk_directfb_modifiers = 0;
int             _gdk_directfb_mouse_x   = 0;
int             _gdk_directfb_mouse_y   = 0;

gboolean
gdk_device_get_history  (GdkDevice      *device,
                         GdkWindow      *window,
                         guint32         start,
                         guint32         stop,
                         GdkTimeCoord ***events,
                         gint           *n_events)
{
  g_return_val_if_fail (window != NULL, FALSE);
  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
  g_return_val_if_fail (events != NULL, FALSE);
  g_return_val_if_fail (n_events != NULL, FALSE);

  *n_events = 0;
  *events = NULL;

  if (GDK_WINDOW_DESTROYED (window))
    return FALSE;

  if (GDK_IS_CORE (device))
    return FALSE;
  else
    return FALSE;
  //TODODO_gdk_device_get_history (device, window, start, stop, events, n_events);
}

void
gdk_device_get_state (GdkDevice       *device,
                      GdkWindow       *window,
                      gdouble         *axes,
                      GdkModifierType *mask)
{
  g_return_if_fail (device != NULL);

  g_return_if_fail (window != NULL);
  g_return_if_fail (GDK_IS_WINDOW (window));

  if (mask)
    *mask = _gdk_directfb_modifiers;
}

void
gdk_input_init (void)
{
  gdk_input_ignore_core = FALSE;

  gdk_input_devices = NULL;
  gdk_input_devices = g_list_append (gdk_input_devices, &gdk_input_core_info);
}

void
gdk_directfb_mouse_get_info (gint            *x,
                             gint            *y,
                             GdkModifierType *mask)
{
  if (x)
    *x = _gdk_directfb_mouse_x;

  if (y)
    *y = _gdk_directfb_mouse_y;

  if (mask)
    *mask = _gdk_directfb_modifiers;
}

void
gdk_directfb_convert_modifiers (DFBInputDeviceModifierKeys dfbmod)
{
  if (dfbmod & DIMK_ALT)
    _gdk_directfb_modifiers |= GDK_MOD1_MASK;
  else
    _gdk_directfb_modifiers &= ~GDK_MOD1_MASK;

  if (dfbmod & DIMK_ALTGR)
    _gdk_directfb_modifiers |= GDK_MOD2_MASK;
  else
    _gdk_directfb_modifiers &= ~GDK_MOD2_MASK;

  if (dfbmod & DIMK_CTRL)
    _gdk_directfb_modifiers |= GDK_CONTROL_MASK;
  else
    _gdk_directfb_modifiers &= ~GDK_CONTROL_MASK;

  if (dfbmod & DIMK_SHIFT)
    _gdk_directfb_modifiers |= GDK_SHIFT_MASK;
  else
    _gdk_directfb_modifiers &= ~GDK_SHIFT_MASK;
}

guint dfb_keys[DIKC_NUMBER_OF_KEYS] =
{
  0,               /*  DIKC_UNKNOWN    */
  GDK_a,           /*  DIKC_A          */
  GDK_b,           /*  DIKC_B          */
  GDK_c,           /*  DIKC_C          */
  GDK_d,           /*  DIKC_D          */
  GDK_e,           /*  DIKC_E          */
  GDK_f,           /*  DIKC_F          */
  GDK_g,           /*  DIKC_G          */
  GDK_h,           /*  DIKC_H          */
  GDK_i,           /*  DIKC_I          */
  GDK_j,           /*  DIKC_J          */
  GDK_k,           /*  DIKC_K          */
  GDK_l,           /*  DIKC_L          */
  GDK_m,           /*  DIKC_M          */
  GDK_n,           /*  DIKC_N          */
  GDK_o,           /*  DIKC_O          */
  GDK_p,           /*  DIKC_P          */
  GDK_q,           /*  DIKC_Q          */
  GDK_r,           /*  DIKC_R          */
  GDK_s,           /*  DIKC_S          */
  GDK_t,           /*  DIKC_T          */
  GDK_u,           /*  DIKC_U          */
  GDK_v,           /*  DIKC_V          */
  GDK_w,           /*  DIKC_W          */
  GDK_x,           /*  DIKC_X          */
  GDK_y,           /*  DIKC_Y          */
  GDK_z,           /*  DIKC_Z          */
  GDK_0,           /*  DIKC_0          */
  GDK_1,           /*  DIKC_1          */
  GDK_2,           /*  DIKC_2          */
  GDK_3,           /*  DIKC_3          */
  GDK_4,           /*  DIKC_4          */
  GDK_5,           /*  DIKC_5          */
  GDK_6,           /*  DIKC_6          */
  GDK_7,           /*  DIKC_7          */
  GDK_8,           /*  DIKC_8          */
  GDK_9,           /*  DIKC_9          */
  GDK_F1,          /*  DIKC_F1         */
  GDK_F2,          /*  DIKC_F2         */
  GDK_F3,          /*  DIKC_F3         */
  GDK_F4,          /*  DIKC_F4         */
  GDK_F5,          /*  DIKC_F5         */
  GDK_F6,          /*  DIKC_F6         */
  GDK_F7,          /*  DIKC_F7         */
  GDK_F8,          /*  DIKC_F8         */
  GDK_F9,          /*  DIKC_F9         */
  GDK_F10,         /*  DIKC_F10        */
  GDK_F11,         /*  DIKC_F11        */
  GDK_F12,         /*  DIKC_F12        */
  GDK_Escape,      /*  DIKC_ESCAPE     */
  GDK_Left,        /*  DIKC_LEFT       */
  GDK_Right,       /*  DIKC_RIGHT      */
  GDK_Up,          /*  DIKC_UP         */
  GDK_Down,        /*  DIKC_DOWN       */
  GDK_Control_L,   /*  DIKC_CTRL         FIXME: L/R ? */
  GDK_Shift_L,     /*  DIKC_SHIFT        FIXME: L/R ? */
  GDK_Alt_L,       /*  DIKC_ALT          FIXME: L/R ? */
  0,               /*  DIKC_ALTGR        FIXME: ???   */
  GDK_Tab,         /*  DIKC_TAB        */
  GDK_Return,      /*  DIKC_ENTER      */
  GDK_space,       /*  DIKC_SPACE      */
  GDK_BackSpace,   /*  DIKC_BACKSPACE  */
  GDK_Insert,      /*  DIKC_INSERT     */
  GDK_Delete,      /*  DIKC_DELETE     */
  GDK_Home,        /*  DIKC_HOME       */
  GDK_End,         /*  DIKC_END        */
  GDK_Page_Up,     /*  DIKC_PAGEUP     */
  GDK_Page_Down,   /*  DIKC_PAGEDOWN   */
  GDK_Caps_Lock,   /*  DIKC_CAPSLOCK   */
  GDK_Num_Lock,    /*  DIKC_NUMLOCK    */
  GDK_Scroll_Lock, /*  DIKC_SCRLOCK    */
  GDK_Print,       /*  DIKC_PRINT      */
  GDK_Pause,       /*  DIKC_PAUSE      */
  GDK_division,    /*  DIKC_KP_DIV     */
  GDK_multiply,    /*  DIKC_KP_MULT    */
  GDK_minus,       /*  DIKC_KP_MINUS   */
  GDK_plus,        /*  DIKC_KP_PLUS    */
  GDK_Return       /*  DIKC_KP_ENTER   */
};

void
gdk_directfb_translate_keycode (DFBWindowEvent *event,
                                guint          *keycode,
                                guchar         *keyascii)
{
  guchar ascii = 0;
  guint  code = 0;

  g_return_if_fail (event != NULL);

  ascii = event->key_ascii;

  if (ascii > 31  &&  ascii != 127)
    code = ascii;
  else
    if (event->keycode < DIKC_NUMBER_OF_KEYS)
      code = dfb_keys[ event->keycode ];

  if (keycode)
    *keycode = code;

  if (keyascii)
    *keyascii = ascii;
}
